/*--------------------------------------------------------------------------*/
/*   Example: Measure.c                                                     */
/*   Purpose: Initialize a DAQ task with 4 measurement channels from        */
/*            an ECU and display the values in a loop. Terminate by any     */
/*            key.                                                          */
/*      Note: To run this example, first start the ECUMC Demo ECU XCP from  */
/*            the ECUMC Demo ECU XCP example folder. To run this example    */
/*            with CCP, replace XCP with CCP throughout this file and use   */
/*            the ECUMC Demo ECU CCP from the ECUMC Demo ECU CCP example    */
/*            folder.                                                       */
/*--------------------------------------------------------------------------*/

#define _CRT_SECURE_NO_WARNINGS
#define _CRT_NONSTDC_NO_DEPRECATE

#ifdef LINUX_BUILD
   #include <string.h>
   #include <sys/ioctl.h>
   #include <sys/select.h>
   #include <unistd.h>
   #include <termios.h>
#else
   #include <conio.h>
   #include <windows.h>
#endif

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

#include <niemc.h>

#define CFALSE   0
#define CTRUE    1

#ifdef LINUX_BUILD
   #define A2LFILE      "../a2l/XcpECU.a2l"
#else
   #define A2LFILE      "..\\a2l\\XcpECU.a2l"
#endif

#ifdef LINUX_BUILD
   #define Sleep usleep
#endif

#define MYINTERFACE  "XCP:CAN1"
#define XNET         CTRUE   // Enable NI-XNET or NI-CAN, TRUE = NI-XNET , FALSE = NI-CAN
#define MEASNAMES    "OneDimTimeSig,PWM_Signal,Sinus1,Sinus2"
#define N_MEAS       4


mcTypeTaskRef DBRef = 0, ECURef = 0, DAQRef = 0;


/*--------------------------------------------------------------------------*/
/*   Function:   CheckError                                                 */
/*    Purpose:   display a message in case of an ECUMC error and terminate  */
/*--------------------------------------------------------------------------*/
void CheckError (mcTypeStatus Stat, char *Function)
{
   char szErrText[1024], szTitle[128];

   if (Stat == 0)
      return;

   mcStatusToString (Stat, sizeof (szErrText), szErrText);

   sprintf (szTitle, "%s in function %s", Stat & 0x80000000 ? "Error" : "Warning", Function);
   printf ("%s:\n%s\n", szTitle, szErrText);

   if (Stat > 0)
      return;

   printf ("Press RETURN to continue...\n");
   getchar ();

   if (DAQRef != 0)
   {
      mcDAQClear (&DAQRef);
      DAQRef = 0;
   }

   if (ECURef != 0)
   {
      mcECUDeselect (&ECURef);
      ECURef = 0;
   }

   if (DBRef != 0)
   {
      mcDatabaseClose (&DBRef);
      DBRef = 0;
   }

   exit (0);
}

#ifdef LINUX_BUILD

/*--------------------------------------------------------------------------*/
/*   Function:   kbhit                                                      */
/*    Purpose:   check if a key is being pressed                            */
/*--------------------------------------------------------------------------*/
   int kbhit() 
   {
	   static const int STDIN = 0;
	   static int initialized = 0;

	   if(! initialized) 
      {
		   struct termios term;
		   tcgetattr(STDIN, &term);
		   term.c_lflag &= ~ICANON;
		   tcsetattr(STDIN, TCSANOW, &term);
		   setbuf(stdin, NULL);
		   initialized = 1;
	   }

	   int bytesWaiting;
	   ioctl(STDIN, FIONREAD, &bytesWaiting);
	   return bytesWaiting;
   }

#endif


/*--------------------------------------------------------------------------*/
/*   Function:   main                                                       */
/*    Purpose:   the main function for this example                         */
/*--------------------------------------------------------------------------*/
int main (void)
{
   mcTypeStatus Stat;
   mcTypeTimestamp tStart, tDelta;
   f64 fMeas[N_MEAS];
   u32 nSamples, i;
   char Interface[20];

   strcpy(Interface, MYINTERFACE);
   if(XNET)
   {
      // NI-XNET enabled
      strcat (Interface, "@nixnet");
   }

   printf ("Opening %s...\n", A2LFILE);
   Stat = mcDatabaseOpen (A2LFILE, &DBRef);
   CheckError (Stat, "mcDatabaseOpen");
   Stat = mcECUSelectEx (DBRef, NULL, Interface, &ECURef);
   CheckError (Stat, "mcECUSelectEx");
   Stat = mcECUConnect (ECURef);
   CheckError (Stat, "mcECUConnect");

   printf ("Starting measurement of\n%s ...\nPress RETURN to terminate...\n\n", MEASNAMES);
   Stat = mcDAQInitialize (MEASNAMES, ECURef, mcDAQModeDAQList, -1, 0.0, &DAQRef);
   CheckError (Stat, "mcDAQInitialize");
   Stat = mcDAQStartStop (DAQRef, mcStartStopModeStart);
   CheckError (Stat, "mcDAQStartStop");

   while (! kbhit ())
   {
      Stat = mcDAQRead (DAQRef, 1, &tStart, &tDelta, fMeas, &nSamples);
      if (Stat != mcWarningOldData && Stat != 0x3FF62009 /* nctWarnOldData */)
         CheckError (Stat, "mcDAQRead");

      for (i=0; i<N_MEAS; i++)
         printf ("%15.6f", fMeas[i]);
      printf ("\n");
      Sleep (50);
   }

   Stat = mcDAQStartStop (DAQRef, mcStartStopModeStop);
   CheckError (Stat, "mcDAQStartStop");
   Stat = mcDAQClear (&DAQRef);
   CheckError (Stat, "mcDAQClear");
   DAQRef = 0;

   Stat = mcECUDisconnect (ECURef);
   CheckError (Stat, "mcECUDisconnect");
   Stat = mcECUDeselect (&ECURef);
   CheckError (Stat, "mcECUDeselect");
   ECURef = 0;
   Stat = mcDatabaseClose (&DBRef);
   CheckError (Stat, "mcDatabaseClose");
   DBRef = 0;

   printf ("\nPress RETURN to continue...\n");
   getchar ();

   return 0;
}
