/*--------------------------------------------------------------------------*/
/*   Example: Parameter.c                                                   */
/*   Purpose: Read a 0D characteristic (parameter value) and allow the      */
/*            user to modify it in a loop. Termination by an invalid        */
/*            value.                                                        */
/*      Note: To run this example, first start the ECUMC Demo ECU XCP from  */
/*            the ECUMC Demo ECU XCP example folder. To run this example    */
/*            with CCP, replace XCP with CCP throughout this file and use   */
/*            the ECUMC Demo ECU CCP from the ECUMC Demo ECU CCP example    */
/*            folder.                                                       */
/*--------------------------------------------------------------------------*/

#define _CRT_SECURE_NO_WARNINGS
#define _CRT_NONSTDC_NO_DEPRECATE

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <niemc.h>

#define CFALSE   0
#define CTRUE    1

#ifdef LINUX_BUILD
   #define A2LFILE      "../a2l/XcpECU.a2l"
#else
   #define A2LFILE      "..\\a2l\\XcpECU.a2l"
#endif

#define MYINTERFACE  "XCP:CAN1"
#define XNET         CTRUE   // Enable NI-XNET or NI-CAN, TRUE = NI-XNET , FALSE = NI-CAN
#define PARNAME      "PWM_Level"


mcTypeTaskRef DBRef = 0, ECURef = 0;


/*--------------------------------------------------------------------------*/
/*   Function:   CheckError                                                 */
/*    Purpose:   display a message in case of an ECUMC error and terminate  */
/*--------------------------------------------------------------------------*/
void CheckError (mcTypeStatus Stat, char *Function)
{
   char szErrText[1024], szTitle[128];

   if (Stat == 0)
      return;

   mcStatusToString (Stat, sizeof (szErrText), szErrText);

   sprintf (szTitle, "%s in function %s", Stat & 0x80000000 ? "Error" : "Warning", Function);
   printf ("%s:\n%s\n", szTitle, szErrText);

   if (Stat > 0)
      return;

   printf ("Press RETURN to continue...\n");
   getchar ();

   if (ECURef != 0)
   {
      mcECUDeselect (&ECURef);
      ECURef = 0;
   }

   if (DBRef != 0)
   {
      mcDatabaseClose (&DBRef);
      DBRef = 0;
   }

   exit (0);
}


/*--------------------------------------------------------------------------*/
/*   Function:   main                                                       */
/*    Purpose:   the main function for this example                         */
/*--------------------------------------------------------------------------*/
int main (void)
{
   mcTypeStatus Stat;
   u32 nDim;
   f64 fMin, fMax, fPar;
   char Interface[20];

   strcpy(Interface, MYINTERFACE);
   if(XNET)
   {
      // NI-XNET enabled
      strcat (Interface, "@nixnet");
   }

   printf ("Opening %s...\n", A2LFILE);
   Stat = mcDatabaseOpen (A2LFILE, &DBRef);
   CheckError (Stat, "mcDatabaseOpen");
   Stat = mcECUSelectEx (DBRef, NULL, Interface, &ECURef);
   CheckError (Stat, "mcECUSelectEx");
   Stat = mcECUConnect (ECURef);
   CheckError (Stat, "mcECUConnect");

   printf ("Checking %s...\n", PARNAME);
   Stat = mcGetProperty (ECURef, PARNAME, mcPropChar_Dimension,
                         sizeof (nDim), &nDim);
   CheckError (Stat, "mcGetProperty");
   if (nDim != 0)
   {
      printf ("Wrong dimension, 0D (parameter) expected!\nPress RETURN to exit...\n");
      getchar ();
      exit (0);
   }

   Stat = mcGetProperty (ECURef, PARNAME, mcPropChar_Minimum,
                         sizeof (fMin), &fMin);
   CheckError (Stat, "mcGetProperty");
   Stat = mcGetProperty (ECURef, PARNAME, mcPropChar_Maximum,
                         sizeof (fMax), &fMax);
   CheckError (Stat, "mcGetProperty");
   printf ("Modifying %s...\nEnter values between %f and %f or 'x' to exit...\n", PARNAME, fMin, fMax);

   for (;;)
   {
      Stat = mcCharacteristicRead (ECURef, PARNAME, &fPar, 1);
      CheckError (Stat, "mcCharacteristicRead");

      printf ("Current Value: %f ...", fPar);
      if (scanf ("%lf", &fPar) != 1 || fPar < fMin || fPar > fMax)
         break;

      Stat = mcCharacteristicWrite (ECURef, PARNAME, &fPar, 1);
      CheckError (Stat, "mcCharacteristicWrite");
   }

   Stat = mcECUDisconnect (ECURef);
   CheckError (Stat, "mcECUDisconnect");
   Stat = mcECUDeselect (&ECURef);
   CheckError (Stat, "mcECUDeselect");
   ECURef = 0;
   Stat = mcDatabaseClose (&DBRef);
   CheckError (Stat, "mcDatabaseClose");
   DBRef = 0;

   printf ("\nPress RETURN to continue...");
   getchar ();

   return 0;
}
